@@ -34,7 +34,7 @@
 #include <complex>
 #include <vector>
 
-#include <string.h>
+#include <cstring>
 
 #include <vips/vips.h>
 
@@ -160,6 +160,11 @@ public:
 		return( vobject ); 
 	}
 
+	bool is_null() const
+	{
+		return vobject == 0;
+	}
+
 };
 
 class VIPS_CPLUSPLUS_API VImage;
@@ -237,6 +242,8 @@ public:
 class VImage : VObject
 {
 public:
+	using VObject::is_null;
+
 	VImage( VipsImage *image, VSteal steal = STEAL ) : 
 		VObject( (VipsObject *) image, steal )
 	{
@@ -320,6 +327,12 @@ public:
 		return( vips_image_get_yoffset( get_image() ) ); 
 	}
 
+	bool
+	has_alpha() const
+	{
+		return( vips_image_hasalpha( get_image() ) );
+	}
+
 	const char *
 	filename() const
 	{
@@ -400,7 +413,7 @@ public:
 	const void *
 	get_blob( const char *field, size_t *length ) const
 	{
-		void *value; 
+		const void *value; 
 
 		if( vips_image_get_blob( this->get_image(), field, 
 			&value, length ) )
@@ -409,6 +422,12 @@ public:
 		return( value ); 
 	}
 
+	bool
+	remove( const char *name ) const
+	{
+		return( vips_image_remove( get_image(), name ) );
+	}
+
 	static VOption *
 	option()
 	{
@@ -451,7 +470,10 @@ public:
 		return( VImage( image ) ); 
 	}
 
-	static VImage new_from_buffer( void *buf, size_t len,
+	static VImage new_from_buffer( const void *buf, size_t len,
+		const char *option_string, VOption *options = 0 );
+
+	static VImage new_from_buffer( const std::string &buf,
 		const char *option_string, VOption *options = 0 );
 
 	static VImage new_matrix( int width, int height );
@@ -488,6 +510,17 @@ public:
 		return( new_from_image( to_vectorv( 1, pixel ) ) ); 
 	}
 
+	VImage 
+	copy_memory() const
+	{
+		VipsImage *image;
+
+		if( !(image = vips_image_copy_memory( this->get_image() )) )
+			throw( VError() );
+
+		return( VImage( image ) );
+	}
+
 	VImage write( VImage out ) const;
 
 	void write_to_file( const char *name, VOption *options = 0 ) const;
@@ -621,6 +654,24 @@ public:
 		return( round( VIPS_OPERATION_ROUND_RINT, options ) ); 
 	}
 
+	VImage 
+	bandand( VOption *options = 0 ) const
+	{
+		return( bandbool( VIPS_OPERATION_BOOLEAN_AND, options ) ); 
+	}
+
+	VImage 
+	bandor( VOption *options = 0 ) const
+	{
+		return( bandbool( VIPS_OPERATION_BOOLEAN_OR, options ) ); 
+	}
+
+	VImage 
+	bandeor( VOption *options = 0 ) const
+	{
+		return( bandbool( VIPS_OPERATION_BOOLEAN_EOR, options ) ); 
+	}
+
 	VImage 
 	real( VOption *options = 0 ) const
 	{
