@@ -158,7 +158,7 @@ show_values( ExifData *data )
  * their default value and we won't know about it. 
  */
 static ExifData *
-vips_exif_load_data_without_fix( void *data, int length )
+vips_exif_load_data_without_fix( const void *data, int length )
 {
 	ExifData *ed;
 
@@ -237,14 +237,25 @@ vips_exif_get_double( ExifData *ed,
 {
 	ExifRational rv;
 	ExifSRational srv;
+	double value;
 
-	if( !vips_exif_get_rational( ed, entry, component, &rv ) ) 
-		*out = (double) rv.numerator / rv.denominator;
-	else if( !vips_exif_get_srational( ed, entry, component, &srv ) ) 
-		*out = (double) srv.numerator / srv.denominator;
+	if( !vips_exif_get_rational( ed, entry, component, &rv ) ) {
+		if( rv.denominator == 0 )
+			value = 0;
+		else
+			value = (double) rv.numerator / rv.denominator;
+	}
+	else if( !vips_exif_get_srational( ed, entry, component, &srv ) ) {
+		if( srv.denominator == 0 )
+			value = 0;
+		else
+			value = (double) srv.numerator / srv.denominator;
+	}
 	else
 		return( -1 );
 
+	*out = value;
+
 	return( 0 );
 }
 
@@ -447,22 +458,6 @@ vips_image_resolution_from_exif( VipsImage *image, ExifData *ed )
 	return( 0 );
 }
 
-static int
-vips_exif_get_thumbnail( VipsImage *im, ExifData *ed )
-{
-	if( ed->size > 0 ) {
-		char *thumb_copy;
-
-		thumb_copy = g_malloc( ed->size );      
-		memcpy( thumb_copy, ed->data, ed->size );
-
-		vips_image_set_blob( im, "jpeg-thumbnail-data", 
-			(VipsCallbackFn) g_free, thumb_copy, ed->size );
-	}
-
-	return( 0 );
-}
-
 /* Need to fwd ref this.
  */
 static int
@@ -474,17 +469,17 @@ vips_exif_resolution_from_image( ExifData *ed, VipsImage *image );
 int
 vips__exif_parse( VipsImage *image )
 {
-	void *data;
-	size_t length;
+	const void *data;
+	size_t size;
 	ExifData *ed;
 	VipsExifParams params;
 	const char *str;
 
 	if( !vips_image_get_typeof( image, VIPS_META_EXIF_NAME ) )
 		return( 0 );
-	if( vips_image_get_blob( image, VIPS_META_EXIF_NAME, &data, &length ) )
+	if( vips_image_get_blob( image, VIPS_META_EXIF_NAME, &data, &size ) )
 		return( -1 ); 
-	if( !(ed = vips_exif_load_data_without_fix( data, length )) )
+	if( !(ed = vips_exif_load_data_without_fix( data, size )) )
 		return( -1 );
 
 #ifdef DEBUG_VERBOSE
@@ -516,7 +511,9 @@ vips__exif_parse( VipsImage *image )
 	exif_data_foreach_content( ed, 
 		(ExifDataForeachContentFunc) vips_exif_get_content, &params );
 
-	vips_exif_get_thumbnail( image, ed );
+	vips_image_set_blob_copy( image, 
+		"jpeg-thumbnail-data", ed->data, ed->size );
+
 	exif_data_free( ed );
 
 	/* Orientation handling. ifd0 has the Orientation tag for the main
@@ -1069,21 +1066,21 @@ vips_exif_set_thumbnail( ExifData *ed, VipsImage *im )
 	/* Update EXIF thumbnail from metadata, if any. 
 	 */
 	if( vips_image_get_typeof( im, "jpeg-thumbnail-data" ) ) { 
-		void *data;
-		size_t length;
+		const void *data;
+		size_t size;
 
 		if( vips_image_get_blob( im, "jpeg-thumbnail-data", 
-			&data, &length ) ) 
+			&data, &size ) ) 
 			return( -1 );
 
 		/* Again, we should use the exif allocator attached to this
 		 * entry, but it is not exposed!
 		 */
-		if( length > 0 && 
+		if( size > 0 && 
 			data ) { 
-			ed->data = malloc( length );
-			memcpy( ed->data, data, length );
-			ed->size = length;
+			ed->data = malloc( size );
+			memcpy( ed->data, data, size );
+			ed->size = size;
 		}
 	}
 
