@@ -51,13 +51,24 @@ int magick_set_image_size( Image *image,
 int magick_import_pixels( Image *image, const ssize_t x, const ssize_t y,
 	const size_t width, const size_t height, const char *map,
 	const StorageType type,const void *pixels, ExceptionInfo *exception );
-void magick_set_property( Image *image, const char *property, const char *value,
+void *magick_images_to_blob( const ImageInfo *image_info, Image *images, 
+	size_t *length, ExceptionInfo *exception );
+void magick_set_property( Image *image, 
+	const char *property, const char *value, ExceptionInfo *exception );
+typedef void *(*MagickMapProfileFn)( Image *image, 
+	const char *name, const void *data, size_t length, void *a );
+void *magick_profile_map( Image *image, MagickMapProfileFn fn, void *a );
+int magick_set_profile( Image *image, 
+	const char *name, const void *data, size_t length, 
 	ExceptionInfo *exception );
+
 void magick_set_image_option( ImageInfo *image_info, 
 	const char *name, const char *value );
 void magick_set_number_scenes( ImageInfo *image_info, 
 	int scene, int number_scenes );
 
+const char *magick_ColorspaceType2str( ColorspaceType colorspace );
+
 ExceptionInfo *magick_acquire_exception( void );
 void magick_destroy_exception( ExceptionInfo *exception );
 void magick_inherit_exception( ExceptionInfo *exception, Image *image );
@@ -69,4 +80,10 @@ void magick_vips_error( const char *domain, ExceptionInfo *exception );
 
 void magick_genesis( void );
 
+int magick_set_vips_profile( VipsImage *im, Image *image );
+int magick_set_magick_profile( Image *image, 
+	VipsImage *im, ExceptionInfo *exception );
+
+gboolean magick_ismagick( const unsigned char *bytes, size_t length );
+
 #endif /*HAVE_MAGICK6*/
