@@ -6,6 +6,8 @@
  * 	- add @n, deprecate @all_frames (just sets n = -1)
  * 24/7/18
  * 	- sniff extra filetypes
+ * 4/2/19
+ * 	- add profile (xmp, ipct, etc.) read
  */
 
 /*
@@ -357,28 +359,28 @@ vips_foreign_load_magick7_class_init( VipsForeignLoadMagick7Class *class )
 		vips_foreign_load_magick7_get_flags_filename;
 	load_class->get_flags = vips_foreign_load_magick7_get_flags;
 
-	VIPS_ARG_STRING( class, "density", 4,
+	VIPS_ARG_STRING( class, "density", 20,
 		_( "Density" ),
 		_( "Canvas resolution for rendering vector formats like SVG" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadMagick7, density ),
 		NULL );
 
-	VIPS_ARG_INT( class, "page", 5,
+	VIPS_ARG_INT( class, "page", 21,
 		_( "Page" ),
 		_( "Load this page from the file" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadMagick7, page ),
 		0, 100000, 0 );
 
-	VIPS_ARG_INT( class, "n", 6,
+	VIPS_ARG_INT( class, "n", 22,
 		_( "n" ),
 		_( "Load this many pages" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadMagick7, n ),
 		-1, 100000, 1 );
 
-	VIPS_ARG_BOOL( class, "all_frames", 7, 
+	VIPS_ARG_BOOL( class, "all_frames", 23, 
 		_( "all_frames" ), 
 		_( "Read all frames from an image" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
@@ -495,8 +497,8 @@ vips_foreign_load_magick7_parse( VipsForeignLoadMagick7 *magick7,
 
 	default:
 		vips_error( class->nickname, 
-			_( "unsupported colorspace %d" ), 
-			(int) image->colorspace );
+			_( "unsupported colorspace %s" ), 
+			magick_ColorspaceType2str( image->colorspace ) );
 		return( -1 );
 	}
 
@@ -527,7 +529,7 @@ vips_foreign_load_magick7_parse( VipsForeignLoadMagick7 *magick7,
 
 	vips_image_pipelinev( out, VIPS_DEMAND_STYLE_SMALLTILE, NULL );
 
-	/* Get all the metadata.
+	/* Get all the string metadata.
 	 */
 	ResetImagePropertyIterator( image );
 	while( (key = GetNextImageProperty( image )) ) {
@@ -544,6 +546,11 @@ vips_foreign_load_magick7_parse( VipsForeignLoadMagick7 *magick7,
 		vips_image_set_string( out, vips_buf_all( &name ), value );
 	}
 
+	/* Set vips metadata from ImageMagick profiles.
+	 */
+	if( magick_set_vips_profile( out, image ) )
+		return( -1 );
+
 	magick7->n_pages = GetImageListLength( GetFirstImageInList( image ) );
 #ifdef DEBUG
 	printf( "image has %d pages\n", magick7->n_pages );
@@ -588,7 +595,7 @@ vips_foreign_load_magick7_parse( VipsForeignLoadMagick7 *magick7,
 
 	/* So we can finally set the height.
 	 */
-	if( magick7->n_frames > 1 ) {
+	if( vips_object_argument_isset( VIPS_OBJECT( magick7 ), "n" ) ) {
 		vips_image_set_int( out, VIPS_META_PAGE_HEIGHT, out->Ysize );
 		out->Ysize *= magick7->n_frames;
 	}
@@ -750,26 +757,15 @@ G_DEFINE_TYPE( VipsForeignLoadMagick7File, vips_foreign_load_magick7_file,
 static gboolean
 ismagick7( const char *filename )
 {
-	Image *image;
-	ImageInfo *image_info;
-	ExceptionInfo *exception;
-	int result;
-
-	magick_genesis();
-
-	/* Horribly slow :-(
+	/* Fetch the first 100 bytes. Hopefully that'll be enough.
 	 */
-	image_info = CloneImageInfo( NULL );
-	exception = magick_acquire_exception();
-	vips_strncpy( image_info->filename, filename, MagickPathExtent );
-	magick_sniff_file( image_info, filename );
-	image = PingImage( image_info, exception );
-	result = image != NULL;
-	VIPS_FREEF( DestroyImageList, image );
-	VIPS_FREEF( DestroyImageInfo, image_info ); 
-	VIPS_FREEF( magick_destroy_exception, exception ); 
+	unsigned char buf[100];
 
-	return( result );
+	/* Files shorter than 100 bytes will leave nonsense at the end of buf,
+	 * but it shouldn't matter.
+	 */
+	return( vips__get_bytes( filename, buf, 100 ) &&
+		magick_ismagick( buf, 100 ) );
 }
 
 static int
@@ -853,25 +849,7 @@ G_DEFINE_TYPE( VipsForeignLoadMagick7Buffer, vips_foreign_load_magick7_buffer,
 static gboolean
 vips_foreign_load_magick7_buffer_is_a_buffer( const void *buf, size_t len )
 {
-	Image *image;
-	ImageInfo *image_info;
-	ExceptionInfo *exception;
-	int result;
-
-	magick_genesis();
-
-	/* Horribly slow :-(
-	 */
-	image_info = CloneImageInfo( NULL );
-	exception = magick_acquire_exception();
-	magick_sniff_bytes( image_info, buf, len );
-	image = PingBlob( image_info, buf, len, exception );
-	result = image != NULL;
-	VIPS_FREEF( DestroyImageList, image );
-	VIPS_FREEF( DestroyImageInfo, image_info ); 
-	VIPS_FREEF( magick_destroy_exception, exception ); 
-
-	return( result );
+	return( magick_ismagick( (const unsigned char *) buf, len ) );
 }
 
 static int
