@@ -183,6 +183,12 @@
  * 	- check for non-byte-multiple bits_per_sample [HongxuChen]
  * 16/8/18
  * 	- shut down the input file as soon as we can [kleisauke]
+ * 28/3/19 omira-sch
+ * 	- better buffer sizing 
+ * 	- ban chroma-subsampled, non-jpg compressed images
+ * 7/6/19
+ * 	- istiff reads the first directory rather than just testing the magic
+ * 	  number, so it ignores more TIFF-like, but not TIFF images
  */
 
 /*
@@ -247,25 +253,38 @@ typedef struct _RtiffHeader {
 	int sample_format;
 	gboolean separate; 
 	int orientation; 
+	gboolean premultiplied;
+	uint16 compression;
 
 	/* Result of TIFFIsTiled().
 	 */
 	gboolean tiled;
 
-	/* Fields for tiled images.
+	/* Fields for tiled images, as returned by libtiff.
 	 */
 	uint32 tile_width;
 	uint32 tile_height;		
+	tsize_t tile_size;
+	tsize_t tile_row_size;
 
-	/* Fields for strip images.
-	 *
-	 * If read_scanlinewise is TRUE, the strips are too large to read in a
-	 * single lump and we need to use the scanline API.
+	/* Fields for strip images, as returned by libtiff.
 	 */
 	uint32 rows_per_strip;
 	tsize_t strip_size;
+	tsize_t scanline_size;
 	int number_of_strips;
+
+	/* If read_scanlinewise is TRUE, the strips are too large to read in a
+	 * single lump and we will use the scanline API.
+	 */
 	gboolean read_scanlinewise;
+
+	/* Strip read geometry. Number of lines we read at once (whole strip
+	 * or 1) and size of the buffer we read to (a scanline, or a strip in
+	 * size).
+	 */
+	uint32 read_height;
+	tsize_t read_size;
 } RtiffHeader;
 
 /* Scanline-type process function.
@@ -319,6 +338,10 @@ typedef struct _Rtiff {
 	 * strips or tiles interleaved. 
 	 */
 	tdata_t contig_buf;
+
+	/* The Y we are reading at. Used to verify strip read is sequential.
+	 */
+	int y_pos;
 } Rtiff;
 
 /* Test for field exists.
@@ -492,6 +515,7 @@ rtiff_new( VipsImage *out, int page, int n, gboolean autorotate )
 	rtiff->memcpy = FALSE;
 	rtiff->plane_buf = NULL;
 	rtiff->contig_buf = NULL;
+	rtiff->y_pos = 0;
 
 	g_signal_connect( out, "close", 
 		G_CALLBACK( rtiff_close_cb ), rtiff ); 
@@ -1130,7 +1154,7 @@ rtiff_parse_palette( Rtiff *rtiff, VipsImage *out )
 	/* Old-style colourmaps were 8-bit. If all the top bytes are zero,
 	 * assume we have one of these.
 	 *
-	 * See: https://github.com/jcupitt/libvips/issues/220
+	 * See: https://github.com/libvips/libvips/issues/220
 	 */
 	for( i = 0; i < len; i++ ) 
 		if( (read->red16[i] >> 8) | 
@@ -1256,7 +1280,12 @@ rtiff_parse_copy( Rtiff *rtiff, VipsImage *out )
 
 	rtiff->sfn = rtiff_memcpy_line;
 	rtiff->client = out;
-	rtiff->memcpy = TRUE;
+
+	/* We expand YCBCR images to RGB using JPEGCOLORMODE_RGB, and this
+	 * means we need a slightly larger read buffer for the edge pixels. In
+	 * turn, this means we can't just memcpy to libvips regions.
+	 */
+	rtiff->memcpy = photometric_interpretation != PHOTOMETRIC_YCBCR;
 
 	return( 0 );
 }
@@ -1291,14 +1320,6 @@ rtiff_pick_reader( Rtiff *rtiff )
 	if( photometric_interpretation == PHOTOMETRIC_PALETTE ) 
 		return( rtiff_parse_palette ); 
 
-	if( photometric_interpretation == PHOTOMETRIC_YCBCR ) { 
-		/* Sometimes JPEG in TIFF images are tagged as YCBCR. Ask
-		 * libtiff to convert to RGB for us.
-		 */
-		TIFFSetField( rtiff->tiff, 
-			TIFFTAG_JPEGCOLORMODE, JPEGCOLORMODE_RGB );
-	}
-
 	return( rtiff_parse_copy );
 }
 
@@ -1311,6 +1332,13 @@ rtiff_set_header( Rtiff *rtiff, VipsImage *out )
 	uint32 data_length;
 	void *data;
 
+	/* Request YCbCr expansion. libtiff complains if you do this for
+	 * non-jpg images.
+	 */
+	if( rtiff->header.compression == COMPRESSION_JPEG )
+		TIFFSetField( rtiff->tiff, 
+			TIFFTAG_JPEGCOLORMODE, JPEGCOLORMODE_RGB );
+
 	out->Xsize = rtiff->header.width;
 	out->Ysize = rtiff->header.height * rtiff->n;
 
@@ -1346,74 +1374,38 @@ rtiff_set_header( Rtiff *rtiff, VipsImage *out )
 	/* Read any ICC profile.
 	 */
 	if( TIFFGetField( rtiff->tiff, 
-		TIFFTAG_ICCPROFILE, &data_length, &data ) &&
-		data &&
-		data_length ) {
-		void *data_copy;
-
-		if( !(data_copy = vips_malloc( NULL, data_length )) ) 
-			return( -1 );
-		memcpy( data_copy, data, data_length );
-		vips_image_set_blob( out, VIPS_META_ICC_NAME, 
-			(VipsCallbackFn) vips_free, data_copy, data_length );
+		TIFFTAG_ICCPROFILE, &data_length, &data ) ) {
+		vips_image_set_blob_copy( out, 
+			VIPS_META_ICC_NAME, data, data_length );
 	}
 
 	/* Read any XMP metadata.
 	 */
 	if( TIFFGetField( rtiff->tiff, 
-		TIFFTAG_XMLPACKET, &data_length, &data ) &&
-		data &&
-		data_length ) {
-		void *data_copy;
-
-		if( !(data_copy = vips_malloc( NULL, data_length )) ) 
-			return( -1 );
-		memcpy( data_copy, data, data_length );
-		vips_image_set_blob( out, VIPS_META_XMP_NAME, 
-			(VipsCallbackFn) vips_free, data_copy, data_length );
+		TIFFTAG_XMLPACKET, &data_length, &data ) ) {
+		vips_image_set_blob_copy( out, 
+			VIPS_META_XMP_NAME, data, data_length );
 	}
 
 	/* Read any IPTC metadata.
 	 */
 	if( TIFFGetField( rtiff->tiff, 
-		TIFFTAG_RICHTIFFIPTC, &data_length, &data ) &&
-		data &&
-		data_length ) {
-		void *data_copy;
-
-		/* libtiff stores IPTC as an array of long, not byte.
-		 */
-		data_length *= 4;
-
-		if( !(data_copy = vips_malloc( NULL, data_length )) ) 
-			return( -1 );
-		memcpy( data_copy, data, data_length );
-		vips_image_set_blob( out, VIPS_META_IPTC_NAME, 
-			(VipsCallbackFn) vips_free, data_copy, data_length );
+		TIFFTAG_RICHTIFFIPTC, &data_length, &data ) ) {
+		vips_image_set_blob_copy( out, 
+			VIPS_META_IPTC_NAME, data, data_length );
 
 		/* Older versions of libvips used this misspelt name :-( attach 
 		 * under this name too for compatibility.
 		 */
-		if( !(data_copy = vips_malloc( NULL, data_length )) ) 
-			return( -1 );
-		memcpy( data_copy, data, data_length );
-		vips_image_set_blob( out, "ipct-data", 
-			(VipsCallbackFn) vips_free, data_copy, data_length );
+		vips_image_set_blob_copy( out, "ipct-data", data, data_length );
 	}
 
 	/* Read any photoshop metadata.
 	 */
 	if( TIFFGetField( rtiff->tiff, 
-		TIFFTAG_PHOTOSHOP, &data_length, &data ) &&
-		data &&
-		data_length ) {
-		void *data_copy;
-
-		if( !(data_copy = vips_malloc( NULL, data_length )) ) 
-			return( -1 );
-		memcpy( data_copy, data, data_length );
-		vips_image_set_blob( out, VIPS_META_PHOTOSHOP_NAME, 
-			(VipsCallbackFn) vips_free, data_copy, data_length );
+		TIFFTAG_PHOTOSHOP, &data_length, &data ) ) {
+		vips_image_set_blob_copy( out, 
+			VIPS_META_PHOTOSHOP_NAME, data, data_length );
 	}
 
 	/* IMAGEDESCRIPTION often has useful metadata.
@@ -1438,19 +1430,6 @@ rtiff_set_header( Rtiff *rtiff, VipsImage *out )
 	return( 0 );
 }
 
-/* The size of the buffer written by TIFFReadTile(). We can't use 
- * TIFFTileSize() since that ignores the setting of TIFFTAG_JPEGCOLORMODE. If
- * this pseudo tag has been set and the tile is encoded with YCbCr, the tile
- * is returned with chrominance upsampled. 
- *
- * This seems not to happen for old-style jpeg-compressed tiles. 
- */
-static size_t
-rtiff_tile_size( Rtiff *rtiff )
-{
-	return( TIFFTileRowSize( rtiff->tiff ) * rtiff->header.tile_height );
-}
-
 /* Allocate a tile buffer. Have one of these for each thread so we can unpack
  * to vips in parallel.
  */
@@ -1458,11 +1437,9 @@ static void *
 rtiff_seq_start( VipsImage *out, void *a, void *b )
 {
 	Rtiff *rtiff = (Rtiff *) a;
-	tsize_t size;
 	tdata_t *buf;
 
-	size = rtiff_tile_size( rtiff );
-	if( !(buf = vips_malloc( NULL, size )) )
+	if( !(buf = vips_malloc( NULL, rtiff->header.tile_size )) )
 		return( NULL );
 
 	return( (void *) buf );
@@ -1517,7 +1494,7 @@ rtiff_fill_region_aligned( VipsRegion *out, void *seq, void *a, void *b )
 	return( 0 );
 }
 
-/* Loop over the output region painting in tiles from the file.
+/* Loop over the output region, painting in tiles from the file.
  */
 static int
 rtiff_fill_region( VipsRegion *out, 
@@ -1527,20 +1504,9 @@ rtiff_fill_region( VipsRegion *out,
 	Rtiff *rtiff = (Rtiff *) a;
 	int tile_width = rtiff->header.tile_width;
 	int tile_height = rtiff->header.tile_height;
+	int tile_row_size = rtiff->header.tile_row_size;
 	VipsRect *r = &out->valid;
 
-	/* Sizeof a line of bytes in the TIFF tile.
-	 */
-	int tls = rtiff_tile_size( rtiff ) / tile_height;
-
-	/* Sizeof a pel in the TIFF file. This won't work for formats which
-	 * are <1 byte per pel, like onebit :-( Fortunately, it's only used
-	 * to calculate addresses within a tile and, because we are wrapped in
-	 * vips_tilecache(), we will never have to calculate positions not 
-	 * within a tile.
-	 */
-	int tps = tls / tile_width;
-
 	int x, y, z;
 
 	/* Special case: we are filling a single tile exactly sized to match
@@ -1608,13 +1574,20 @@ rtiff_fill_region( VipsRegion *out,
 			 */
 			vips_rect_intersectrect( &tile, r, &hit );
 
+			/* We are inside a tilecache, so requests will always
+			 * be aligned left-right to tile boundaries.
+			 *
+			 * this is not true vertically for toilet-roll images.
+			 */
+			g_assert( hit.left == tile.left );
+
 			/* Unpack to VIPS format. 
 			 * Just unpack the section of the tile we need.
 			 */
 			for( z = 0; z < hit.height; z++ ) {
 				VipsPel *p = (VipsPel *) buf +
-					(hit.left - tile.left) * tps +
-					(hit.top - tile.top + z) * tls;
+					(hit.top - tile.top + z) * 
+					tile_row_size;
 				VipsPel *q = VIPS_REGION_ADDR( out, 
 					hit.left, hit.top + z );
 
@@ -1687,6 +1660,29 @@ rtiff_autorotate( Rtiff *rtiff, VipsImage *in, VipsImage **out )
 	return( 0 );
 }
 
+/* Unpremultiply associative alpha, if any.
+ */
+static int
+rtiff_unpremultiply( Rtiff *rtiff, VipsImage *in, VipsImage **out )
+{
+	if( rtiff->header.premultiplied ) {
+		VipsImage *x;
+
+		if( vips_unpremultiply( in, &x, NULL ) ||
+			vips_cast( x, out, in->BandFmt, NULL ) ) {
+			g_object_unref( x );
+			return( -1 );
+		}
+		g_object_unref( x );
+	}
+	else {
+		*out = in;
+		g_object_ref( in );
+	}
+
+	return( 0 );
+}
+
 /* Tile-type TIFF reader core - pass in a per-tile transform. Generate into
  * the im and do it all partially.
  */
@@ -1696,7 +1692,7 @@ rtiff_read_tilewise( Rtiff *rtiff, VipsImage *out )
 	int tile_width = rtiff->header.tile_width;
 	int tile_height = rtiff->header.tile_height;
 	VipsImage **t = (VipsImage **) 
-		vips_object_local_array( VIPS_OBJECT( out ), 3 );
+		vips_object_local_array( VIPS_OBJECT( out ), 4 );
 
 #ifdef DEBUG
 	printf( "tiff2vips: rtiff_read_tilewise\n" );
@@ -1721,12 +1717,10 @@ rtiff_read_tilewise( Rtiff *rtiff, VipsImage *out )
 	 * match the tifftile size.
 	 */
 	if( rtiff->memcpy ) {
-		size_t vips_tile_size;
-
-		vips_tile_size = VIPS_IMAGE_SIZEOF_PEL( t[0] ) * 
+		size_t vips_tile_size = VIPS_IMAGE_SIZEOF_PEL( t[0] ) * 
 			tile_width * tile_height; 
 
-		if( rtiff_tile_size( rtiff ) != vips_tile_size ) { 
+		if( rtiff->header.tile_size != vips_tile_size ) { 
 			vips_error( "tiff2vips", 
 				"%s", _( "unsupported tiff image type" ) );
 			return( -1 );
@@ -1750,11 +1744,10 @@ rtiff_read_tilewise( Rtiff *rtiff, VipsImage *out )
 		"tile_width", tile_width,
 		"tile_height", tile_height,
 		"max_tiles", 2 * (1 + t[0]->Xsize / tile_width),
-		NULL ) ) 
-		return( -1 );
-	if( rtiff_autorotate( rtiff, t[1], &t[2] ) )
-		return( -1 );
-	if( vips_image_write( t[2], out ) ) 
+		NULL ) ||
+		rtiff_autorotate( rtiff, t[1], &t[2] ) ||
+		rtiff_unpremultiply( rtiff, t[2], &t[3] ) ||
+		vips_image_write( t[3], out ) )
 		return( -1 );
 
 	return( 0 );
@@ -1769,15 +1762,15 @@ static int
 rtiff_strip_read_interleaved( Rtiff *rtiff, tstrip_t strip, tdata_t buf )
 {
 	int samples_per_pixel = rtiff->header.samples_per_pixel;
-	int rows_per_strip = rtiff->header.rows_per_strip;
+	int read_height = rtiff->header.read_height;
 	int bits_per_sample = rtiff->header.bits_per_sample;
-	int strip_y = strip * rows_per_strip;
+	int strip_y = strip * read_height;
 
 	if( rtiff->header.separate ) {
 		int page_width = rtiff->header.width;
 		int page_height = rtiff->header.height;
-		int strips_per_plane = 1 + (page_height - 1) / rows_per_strip;
-		int strip_height = VIPS_MIN( rows_per_strip, 
+		int strips_per_plane = 1 + (page_height - 1) / read_height;
+		int strip_height = VIPS_MIN( read_height, 
 			page_height - strip_y ); 
 		int pels_per_strip = page_width * strip_height;
 		int bytes_per_sample = bits_per_sample >> 3; 
@@ -1817,16 +1810,17 @@ rtiff_stripwise_generate( VipsRegion *or,
 	void *seq, void *a, void *b, gboolean *stop )
 {
 	Rtiff *rtiff = (Rtiff *) a;
-	int rows_per_strip = rtiff->header.rows_per_strip;
+	int read_height = rtiff->header.read_height;
 	int page_height = rtiff->header.height;
-	tsize_t scanline_size = TIFFScanlineSize( rtiff->tiff );
+	tsize_t scanline_size = rtiff->header.scanline_size;
         VipsRect *r = &or->valid;
 
 	int y;
 
 #ifdef DEBUG
-	printf( "tiff2vips: read_stripwise_generate: top = %d, height = %d\n",
+	printf( "rtiff_stripwise_generate: top = %d, height = %d\n",
 		r->top, r->height );
+	printf( "rtiff_stripwise_generate: y_top = %d\n", rtiff->y_pos );
 #endif /*DEBUG*/
 
 	/* We're inside a tilecache where tiles are the full image width, so
@@ -1844,7 +1838,16 @@ rtiff_stripwise_generate( VipsRegion *or,
 	 * strip in the image.
 	 */
 	g_assert( r->height == 
-		VIPS_MIN( rows_per_strip, or->im->Ysize - r->top ) ); 
+		VIPS_MIN( read_height, or->im->Ysize - r->top ) ); 
+
+	/* And check that y_pos is correct. It should be, since we are inside
+	 * a vips_sequential().
+	 */
+	if( r->top != rtiff->y_pos ) {
+		vips_error( "tiff2vips", 
+			_( "out of order read at line %d" ), rtiff->y_pos );
+		return( -1 );
+	}
 
 	VIPS_GATE_START( "rtiff_stripwise_generate: work" ); 
 
@@ -1859,7 +1862,7 @@ rtiff_stripwise_generate( VipsRegion *or,
 
 		/* Strip number.
 		 */
-		tstrip_t strip_no = y_page / rows_per_strip;
+		tstrip_t strip_no = y_page / read_height;
 
 		VipsRect image, page, strip, hit;
 
@@ -1877,9 +1880,9 @@ rtiff_stripwise_generate( VipsRegion *or,
 		page.height = page_height;
 
 		strip.left = 0;
-		strip.top = page.top + strip_no * rows_per_strip;
+		strip.top = page.top + strip_no * read_height;
 		strip.width = rtiff->out->Xsize;
-		strip.height = rows_per_strip;
+		strip.height = read_height;
 
 		/* Clip strip against page and image ... the final strip will 
 		 * be smaller.
@@ -1943,12 +1946,17 @@ rtiff_stripwise_generate( VipsRegion *or,
 		}
 
 		y += hit.height;
+		rtiff->y_pos += hit.height;
 	}
 
 	/* Shut down the input file as soon as we can. 
 	 */
-	if( r->top + y >= or->im->Ysize ) 
+	if( rtiff->y_pos >= or->im->Ysize ) {
+#ifdef DEBUG
+		printf( "rtiff_stripwise_generate: early shutdown\n" ); 
+#endif /*DEBUG*/
 		rtiff_free( rtiff );
+	}
 
 	VIPS_GATE_STOP( "rtiff_stripwise_generate: work" ); 
 
@@ -1965,7 +1973,7 @@ static int
 rtiff_read_stripwise( Rtiff *rtiff, VipsImage *out )
 {
 	VipsImage **t = (VipsImage **) 
-		vips_object_local_array( VIPS_OBJECT( out ), 3 );
+		vips_object_local_array( VIPS_OBJECT( out ), 4 );
 
 #ifdef DEBUG
 	printf( "tiff2vips: rtiff_read_stripwise\n" );
@@ -1984,6 +1992,10 @@ rtiff_read_stripwise( Rtiff *rtiff, VipsImage *out )
 		rtiff->header.strip_size );
 	printf( "rtiff_read_stripwise: header.number_of_strips = %d\n", 
 		rtiff->header.number_of_strips );
+	printf( "rtiff_read_stripwise: header.read_height = %u\n", 
+		rtiff->header.read_height );
+	printf( "rtiff_read_stripwise: header.read_size = %zd\n", 
+		rtiff->header.read_size );
 #endif /*DEBUG*/
 
 	/* Double check: in memcpy mode, the vips linesize should exactly
@@ -2000,7 +2012,7 @@ rtiff_read_stripwise( Rtiff *rtiff, VipsImage *out )
 		else
 			vips_line_size = VIPS_IMAGE_SIZEOF_LINE( t[0] );
 
-		if( vips_line_size != TIFFScanlineSize( rtiff->tiff ) ) { 
+		if( vips_line_size != rtiff->header.scanline_size ) { 
 			vips_error( "tiff2vips", 
 				"%s", _( "unsupported tiff image type" ) );
 			return( -1 );
@@ -2015,7 +2027,7 @@ rtiff_read_stripwise( Rtiff *rtiff, VipsImage *out )
 	 */
 	if( rtiff->header.separate ) {
 		if( !(rtiff->plane_buf = vips_malloc( VIPS_OBJECT( out ), 
-			rtiff->header.strip_size )) ) 
+			rtiff->header.read_size )) ) 
 			return( -1 );
 	}
 
@@ -2032,25 +2044,28 @@ rtiff_read_stripwise( Rtiff *rtiff, VipsImage *out )
 		rtiff->n > 1 ) { 
 		tsize_t size;
 
-		size = rtiff->header.strip_size;
+		size = rtiff->header.read_size;
 		if( rtiff->header.separate )
 			size *= rtiff->header.samples_per_pixel;
 
 		if( !(rtiff->contig_buf = 
 			vips_malloc( VIPS_OBJECT( out ), size )) ) 
 			return( -1 );
-
 	}
 
+	/* rows_per_strip can be very large if this is a separate plane image,
+	 * beware.
+	 */
 	if( 
 		vips_image_generate( t[0], 
 			NULL, rtiff_stripwise_generate, NULL, 
 			rtiff, NULL ) ||
 		vips_sequential( t[0], &t[1], 
-			"tile_height", rtiff->header.rows_per_strip,
+			"tile_height", rtiff->header.read_height,
 			NULL ) ||
 		rtiff_autorotate( rtiff, t[1], &t[2] ) ||
-		vips_image_write( t[2], out ) )
+		rtiff_unpremultiply( rtiff, t[2], &t[3] ) ||
+		vips_image_write( t[3], out ) )
 		return( -1 );
 
 	return( 0 );
@@ -2061,6 +2076,9 @@ rtiff_read_stripwise( Rtiff *rtiff, VipsImage *out )
 static int
 rtiff_header_read( Rtiff *rtiff, RtiffHeader *header )
 {
+	uint16 extra_samples_count;
+	uint16 *extra_samples_types;
+
 	if( !tfget32( rtiff->tiff, TIFFTAG_IMAGEWIDTH, &header->width ) ||
 		!tfget32( rtiff->tiff, TIFFTAG_IMAGELENGTH, &header->height ) ||
 		!tfget16( rtiff->tiff, 
@@ -2072,13 +2090,40 @@ rtiff_header_read( Rtiff *rtiff, RtiffHeader *header )
 			&header->photometric_interpretation ) )
 		return( -1 );
 
+	TIFFGetFieldDefaulted( rtiff->tiff, 
+		TIFFTAG_COMPRESSION, &header->compression );
+	if( header->compression == COMPRESSION_JPEG )
+		/* We want to always expand subsampled YCBCR images to full 
+		 * RGB. 
+		 */
+		TIFFSetField( rtiff->tiff, 
+			TIFFTAG_JPEGCOLORMODE, JPEGCOLORMODE_RGB );
+        else if( header->photometric_interpretation == PHOTOMETRIC_YCBCR ) {
+		/* We rely on the jpg decompressor to upsample chroma
+		 * subsampled images. If there is chroma subsampling but
+		 * no jpg compression, we have to give up.
+		 *
+		 * tiffcp fails for images like this too.
+		 */
+                uint16 hsub, vsub;
+
+                TIFFGetFieldDefaulted( rtiff->tiff, 
+			TIFFTAG_YCBCRSUBSAMPLING, &hsub, &vsub );
+                if( hsub != 1 || 
+			vsub != 1 ) {
+			vips_error( "tiff2vips",
+				"%s", _( "subsampled images not supported" ) );
+			return( -1 );
+                }
+        }
+
 	/* Arbitrary sanity-checking limits.
 	 */
-	if( header->width <= 0 || 
-		header->width > VIPS_MAX_COORD || 
-		header->height <= 0 || 
+	if( header->width <= 0 ||
+		header->width > VIPS_MAX_COORD ||
+		header->height <= 0 ||
 		header->height > VIPS_MAX_COORD ) {
-		vips_error( "tiff2vips", 
+		vips_error( "tiff2vips",
 			"%s", _( "width/height out of range" ) );
 		return( -1 );
 	}
@@ -2116,27 +2161,35 @@ rtiff_header_read( Rtiff *rtiff, RtiffHeader *header )
 				TIFFTAG_TILELENGTH, &header->tile_height ) )
 			return( -1 );
 
+		/* Arbitrary sanity-checking limits.
+		 */
+		if( header->tile_width <= 0 ||
+			header->tile_width > 10000 ||
+			header->tile_height <= 0 ||
+			header->tile_height > 10000 ) {
+			vips_error( "tiff2vips",
+				"%s", _( "tile size out of range" ) );
+			return( -1 );
+		}
+
+		header->tile_size = TIFFTileSize( rtiff->tiff );
+		header->tile_row_size = TIFFTileRowSize( rtiff->tiff );
+
 		/* Stop some compiler warnings.
 		 */
 		header->rows_per_strip = 0; 
 		header->strip_size = 0; 
 		header->number_of_strips = 0; 
+		header->read_height = 0;
+		header->read_size = 0;
 	}
 	else {
 		if( !tfget32( rtiff->tiff, 
 			TIFFTAG_ROWSPERSTRIP, &header->rows_per_strip ) )
 			return( -1 );
 		header->strip_size = TIFFStripSize( rtiff->tiff );
+		header->scanline_size = TIFFScanlineSize( rtiff->tiff );
 		header->number_of_strips = TIFFNumberOfStrips( rtiff->tiff );
-		header->read_scanlinewise = FALSE;
-
-		/* rows_per_strip can be 2 ** 32 - 1, meaning the whole image. 
-		 * Clip this down to height to avoid confusing vips. 
-		 *
-		 * And it musn't be zero.
-		 */
-		header->rows_per_strip = 
-			VIPS_CLIP( 1, header->rows_per_strip, header->height );
 
 		/* libtiff has two strip-wise readers. TIFFReadEncodedStrip()
 		 * decompresses an entire strip to memory. It's fast, but it
@@ -2149,21 +2202,46 @@ rtiff_header_read( Rtiff *rtiff, RtiffHeader *header )
 		 *
 		 * Don't do this in plane-separate mode. TIFFReadScanline() is
 		 * too fiddly to use in this case.
+		 *
+		 * Don't try scanline reading for YCbCr images.
+		 * TIFFScanlineSize() will not work in this case due to
+		 * chroma subsampling.
 		 */
 		if( header->rows_per_strip > 128 &&
-			!header->separate ) {
-			header->rows_per_strip = 1;
-			header->strip_size = TIFFScanlineSize( rtiff->tiff );
-			header->number_of_strips = header->height;
+			!header->separate &&
+			header->photometric_interpretation != 
+				PHOTOMETRIC_YCBCR ) {
 			header->read_scanlinewise = TRUE;
+			header->read_height = 1;
+			header->read_size = rtiff->header.scanline_size;
+		}
+		else {
+			header->read_scanlinewise = FALSE;
+
+			/* rows_per_strip can be 2 ** 32 - 1, meaning the 
+			 * whole image. Clip this down to height to avoid 
+			 * confusing vips. 
+			 *
+			 * And it musn't be zero.
+			 */
+			header->read_height = VIPS_CLIP( 1, 
+				header->rows_per_strip, header->height );
+			header->read_size = header->strip_size;
 		}
 
 		/* Stop some compiler warnings.
 		 */
 		header->tile_width = 0;
 		header->tile_height = 0;
+		header->tile_size = 0;
+		header->tile_row_size = 0;
 	}
 
+	TIFFGetFieldDefaulted( rtiff->tiff, TIFFTAG_EXTRASAMPLES,
+		&extra_samples_count, &extra_samples_types );
+	header->premultiplied = extra_samples_count > 0 &&
+		extra_samples_types[0] == EXTRASAMPLE_ASSOCALPHA;
+
 	return( 0 );
 }
 
@@ -2188,8 +2266,8 @@ rtiff_header_equal( RtiffHeader *h1, RtiffHeader *h2 )
 			return( 0 );
 	}
 	else {
-		if( h1->rows_per_strip != h2->rows_per_strip ||
-			h1->strip_size != h2->strip_size ||
+		if( h1->read_height != h2->read_height ||
+			h1->read_size != h2->read_size ||
 			h1->number_of_strips != h2->number_of_strips )
 			return( 0 );
 	}
@@ -2273,25 +2351,7 @@ rtiff_new_buffer( const void *buf, size_t len, VipsImage *out,
 	return( rtiff );
 }
 
-/* 
-
-	FIXME ... Unused for now, perhaps if we add another format flag.
-
-static int
-istiffpyramid( const char *name )
-{
-	TIFF *tif;
-
-	vips__tiff_init();
-
-	if( (tif = get_directory( name, 2 )) ) {
-		// We can see page 2 ... assume it is.
-		TIFFClose( tif );
-		return( 1 );
-	}
-
-	return( 0 );
-}
+/* For istiffpyramid(), see vips_thumbnail_get_tiff_pyramid().
  */
 
 int
@@ -2307,7 +2367,8 @@ vips__tiff_read( const char *filename, VipsImage *out,
 
 	vips__tiff_init();
 
-	if( !(rtiff = rtiff_new_filename( filename, out, page, n, autorotate )) )
+	if( !(rtiff = rtiff_new_filename( filename, out, 
+		page, n, autorotate )) )
 		return( -1 );
 
 	if( rtiff->header.tiled ) {
@@ -2322,7 +2383,7 @@ vips__tiff_read( const char *filename, VipsImage *out,
 	return( 0 );
 }
 
-/* On a header-only read, we can just swap width/height if orientaion is 6 or
+/* On a header-only read, we can just swap width/height if orientation is 6 or
  * 8. 
  */
 static void
@@ -2369,11 +2430,13 @@ vips__tiff_read_header( const char *filename, VipsImage *out,
 	return( 0 );
 }
 
-gboolean
-vips__istifftiled( const char *filename )
+typedef gboolean (*TiffPropertyFn)( TIFF *tif );
+
+static gboolean
+vips__testtiff( const char *filename, TiffPropertyFn fn )
 {
 	TIFF *tif;
-	gboolean tiled;
+	gboolean property;
 
 	vips__tiff_init();
 
@@ -2381,37 +2444,59 @@ vips__istifftiled( const char *filename )
 		vips_error_clear();
 		return( FALSE );
 	}
-	tiled = TIFFIsTiled( tif );
+
+	property = fn ? fn( tif ) : TRUE;
+
 	TIFFClose( tif );
 
-	return( tiled );
+	return( property );
+}
+
+gboolean
+vips__testtiff_buffer( const void *buf, size_t len, TiffPropertyFn fn )
+{
+	VipsImage *im;
+	TIFF *tif;
+	gboolean property;
+
+	vips__tiff_init();
+
+	im = vips_image_new();
+
+	if( !(tif = vips__tiff_openin_buffer( im, buf, len )) ) {
+		g_object_unref( im );
+		vips_error_clear();
+		return( FALSE );
+	}
+
+	property = fn ? fn( tif ) : TRUE;
+
+	TIFFClose( tif );
+	g_object_unref( im );
+
+	return( property );
+}
+
+gboolean
+vips__istifftiled( const char *filename )
+{
+	return( vips__testtiff( filename, TIFFIsTiled ) ); 
+}
+
+/* We test for TIFF by trying to read the first directory. We could just test
+ * the magic number, but many formats (eg. ARW) use a TIFF-like container and
+ * we don't want to open those with vips tiffload.
+ */
+gboolean
+vips__istiff( const char *filename )
+{
+	return( vips__testtiff( filename, NULL ) ); 
 }
 
 gboolean
 vips__istiff_buffer( const void *buf, size_t len )
 {
-	char *str = (char *) buf; 
-
-	if( len >= 4 &&
-		((str[0] == 'M' && str[1] == 'M' &&
-			str[2] == '\0' && (str[3] == '*' || str[3] == '+')) ||
-		(str[0] == 'I' && str[1] == 'I' &&
-			(str[2] == '*' || str[2] == '+') && str[3] == '\0')) )
-		return( TRUE );
-
-	return( FALSE );
-}
-
-gboolean
-vips__istiff( const char *filename )
-{
-	unsigned char buf[4];
-
-	if( vips__get_bytes( filename, buf, 4 ) == 4 &&
-		vips__istiff_buffer( buf, 4 ) )
-		return( TRUE );
-
-	return( FALSE );
+	return( vips__testtiff_buffer( buf, len, NULL ) ); 
 }
 
 int
