@@ -4,6 +4,10 @@
  * 	- from pngload.c
  * 28/2/16
  * 	- add @shrink
+ * 1/11/18
+ * 	- add @page, @n
+ * 30/4/19
+ * 	- deprecate @shrink, use @scale instead
  */
 
 /*
@@ -54,9 +58,21 @@
 typedef struct _VipsForeignLoadWebp {
 	VipsForeignLoad parent_object;
 
-	/* Shrink by this much during load.
+	/* Load this page (frame number).
 	 */
-	int shrink; 
+	int page;
+
+	/* Load this many pages.
+	 */
+	int n;
+
+	/* Scale by this much during load.
+	 */
+	double scale; 
+
+	/* Old and deprecated scaling path.
+	 */
+	int shrink;
 } VipsForeignLoadWebp;
 
 typedef VipsForeignLoadClass VipsForeignLoadWebpClass;
@@ -96,10 +112,34 @@ vips_foreign_load_webp_class_init( VipsForeignLoadWebpClass *class )
 
 	load_class->get_flags = vips_foreign_load_webp_get_flags;
 
-	VIPS_ARG_INT( class, "shrink", 10, 
+	VIPS_ARG_INT( class, "page", 20,
+		_( "Page" ),
+		_( "Load this page from the file" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignLoadWebp, page ),
+		0, 100000, 0 );
+
+	VIPS_ARG_INT( class, "n", 21,
+		_( "n" ),
+		_( "Load this many pages" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignLoadWebp, n ),
+		-1, 100000, 1 );
+
+	VIPS_ARG_DOUBLE( class, "scale", 22, 
+		_( "Scale" ), 
+		_( "Scale factor on load" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignLoadWebp, scale ),
+		0.0, 1024.0, 1.0 );
+
+	/* Old and deprecated scaling API. A float param lets do
+	 * shrink-on-load for thumbnail faster and more accurately.
+	 */
+	VIPS_ARG_INT( class, "shrink", 23, 
 		_( "Shrink" ), 
 		_( "Shrink factor on load" ),
-		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
 		G_STRUCT_OFFSET( VipsForeignLoadWebp, shrink ),
 		1, 1024, 1 );
 
@@ -108,7 +148,9 @@ vips_foreign_load_webp_class_init( VipsForeignLoadWebpClass *class )
 static void
 vips_foreign_load_webp_init( VipsForeignLoadWebp *webp )
 {
+	webp->n = 1;
 	webp->shrink = 1;
+	webp->scale = 1.0;
 }
 
 typedef struct _VipsForeignLoadWebpFile {
@@ -143,8 +185,15 @@ vips_foreign_load_webp_file_header( VipsForeignLoad *load )
 	VipsForeignLoadWebp *webp = (VipsForeignLoadWebp *) load;
 	VipsForeignLoadWebpFile *file = (VipsForeignLoadWebpFile *) load;
 
+	/* BC for the old API.
+	 */
+	if( !vips_object_argument_isset( VIPS_OBJECT( load ), "scale" ) &&
+		vips_object_argument_isset( VIPS_OBJECT( load ), "shrink" ) &&
+		webp->shrink != 0 )
+		webp->scale = 1.0 / webp->shrink;
+
 	if( vips__webp_read_file_header( file->filename, load->out, 
-		webp->shrink ) )
+		webp->page, webp->n, webp->scale ) )
 		return( -1 );
 
 	VIPS_SETSTR( load->out->filename, file->filename );
@@ -158,7 +207,8 @@ vips_foreign_load_webp_file_load( VipsForeignLoad *load )
 	VipsForeignLoadWebp *webp = (VipsForeignLoadWebp *) load;
 	VipsForeignLoadWebpFile *file = (VipsForeignLoadWebpFile *) load;
 
-	if( vips__webp_read_file( file->filename, load->real, webp->shrink ) )
+	if( vips__webp_read_file( file->filename, load->real, 
+		webp->page, webp->n, webp->scale ) )
 		return( -1 );
 
 	return( 0 );
@@ -222,7 +272,8 @@ vips_foreign_load_webp_buffer_header( VipsForeignLoad *load )
 	VipsForeignLoadWebpBuffer *buffer = (VipsForeignLoadWebpBuffer *) load;
 
 	if( vips__webp_read_buffer_header( buffer->buf->data, 
-		buffer->buf->length, load->out, webp->shrink ) )
+		buffer->buf->length, load->out, 
+		webp->page, webp->n, webp->scale ) )
 		return( -1 );
 
 	return( 0 );
@@ -235,7 +286,8 @@ vips_foreign_load_webp_buffer_load( VipsForeignLoad *load )
 	VipsForeignLoadWebpBuffer *buffer = (VipsForeignLoadWebpBuffer *) load;
 
 	if( vips__webp_read_buffer( buffer->buf->data, buffer->buf->length, 
-		load->real, webp->shrink ) )
+		load->real, 
+		webp->page, webp->n, webp->scale ) )
 		return( -1 );
 
 	return( 0 );
@@ -287,14 +339,23 @@ vips_foreign_load_webp_buffer_init( VipsForeignLoadWebpBuffer *buffer )
  *
  * Optional arguments:
  *
- * * @shrink: %gint, shrink by this much on load
+ * * @page: %gint, page (frame) to read
+ * * @n: %gint, load this many pages
+ * * @scale: %gdouble, scale by this much on load
  *
  * Read a WebP file into a VIPS image. 
  *
- * Use @shrink to specify a shrink-on-load factor.
+ * Use @page to select a page to render, numbering from zero.
  *
- * If libwebpmux is available, image metadata is also read. The loader supports 
- * ICC, EXIF and XMP metadata. 
+ * Use @n to select the number of pages to render. The default is 1. Pages are
+ * rendered in a vertical column, with each individual page aligned to the
+ * left. Set to -1 to mean "until the end of the document". Use vips_grid() 
+ * to change page layout.
+ *
+ * Use @scale to specify a scale-on-load factor. For example, 2.0 to double
+ * the size on load.
+ *
+ * The loader supports ICC, EXIF and XMP metadata. 
  *
  * See also: vips_image_new_from_file().
  *
@@ -322,7 +383,9 @@ vips_webpload( const char *filename, VipsImage **out, ... )
  *
  * Optional arguments:
  *
- * * @shrink: %gint, shrink by this much on load
+ * * @page: %gint, page (frame) to read
+ * * @n: %gint, load this many pages
+ * * @scale: %gdouble, scale by this much on load
  *
  * Read a WebP-formatted memory block into a VIPS image. Exactly as
  * vips_webpload(), but read from a memory buffer. 
