@@ -26,6 +26,14 @@
  * 	- return header enums as enums, not ints
  * 	- vips_image_get_*() all convert everything to target type if they can
  * 	- rename "field" as "name" in docs
+ * 21/11/18
+ * 	- get_string will allow G_STRING and REF_STRING
+ * 28/12/18
+ * 	- hide deprecated header fields from _map
+ * 17/2/19
+ * 	- add vips_image_get_page_height()
+ * 19/6/19
+ * 	- add vips_image_get_n_pages()
  */
 
 /*
@@ -84,7 +92,7 @@
  * uniform way. 
  *
  * Use vips_image_get_typeof() to test for the 
- * existance and #GType of a header field.
+ * existence and #GType of a header field.
  *
  * You can attach arbitrary metadata to images. Metadata is copied as images
  * are processed, so all images which used this image as input, directly or
@@ -772,6 +780,57 @@ vips_image_get_offset( const VipsImage *image )
 	return( offset );
 }
 
+/**
+ * vips_image_get_page_height: (method)
+ * @image: image to get from
+ *
+ * Multi-page images can have a page height. Fetch it, and sanity check it. If
+ * page-height is not set, it defaults to the image height.
+ *
+ * Returns: the page height.
+ */
+int
+vips_image_get_page_height( VipsImage *image )
+{
+	int page_height;
+
+	if( !vips_image_get_typeof( image, VIPS_META_PAGE_HEIGHT ) ||
+		vips_image_get_int( image, VIPS_META_PAGE_HEIGHT, 
+			&page_height ) ||
+		page_height <= 0 ||
+		page_height > image->Ysize ||
+		image->Ysize % page_height != 0 ) 
+		page_height = image->Ysize;
+
+	return( page_height );
+}
+
+/**
+ * vips_image_get_n_pages: (method)
+ * @image: image to get from
+ *
+ * Fetch and sanity-check VIPS_META_N_PAGES. Default to 1 if not present or
+ * crazy.
+ *
+ * This is the number of pages in the image file, not the number of pages that
+ * have been loaded into @image.
+ *
+ * Returns: the number of pages in the image file
+ */
+int
+vips_image_get_n_pages( VipsImage *image )
+{
+	int n_pages;
+
+	if( !vips_image_get_typeof( image, VIPS_META_N_PAGES ) ||
+		vips_image_get_int( image, VIPS_META_N_PAGES, &n_pages ) ||
+		n_pages < 2 || 
+		n_pages > 1000 )
+		n_pages = 1;
+
+	return( n_pages );
+}
+
 /**
  * vips_image_get_data: (method)
  * @image: image to get data for
@@ -1158,9 +1217,23 @@ vips_image_remove( VipsImage *image, const char *name )
 	return( FALSE );
 }
 
+/* Deprecated header fields we hide from _map.
+ */
+static const char *vips_image_header_deprecated[] = {
+	"ipct-data"
+};
+
 static void *
 vips_image_map_fn( VipsMeta *meta, VipsImageMapFn fn, void *a )
 {
+	int i;
+
+	/* Hide deprecated fields.
+	 */
+	for( i = 0; i < VIPS_NUMBER( vips_image_header_deprecated ); i++ )
+		if( strcmp( meta->name, vips_image_header_deprecated[i] ) == 0 )
+			return( NULL );
+
 	return( fn( meta->im, meta->name, &meta->value, a ) );
 }
 
@@ -1311,7 +1384,7 @@ meta_get_value( const VipsImage *image,
  *
  * Gets @data from @image under the name @name. A convenience
  * function over vips_image_get(). Use vips_image_get_typeof() to test for
- * the existance of a piece of metadata.
+ * the existence of a piece of metadata.
  *
  * See also: vips_image_set_area(), vips_image_get(),
  * vips_image_get_typeof()
@@ -1319,7 +1392,8 @@ meta_get_value( const VipsImage *image,
  * Returns: 0 on success, -1 otherwise.
  */
 int
-vips_image_get_area( const VipsImage *image, const char *name, void **data )
+vips_image_get_area( const VipsImage *image, 
+	const char *name, const void **data )
 {
 	GValue value_copy = { 0 };
 
@@ -1348,16 +1422,55 @@ vips_image_get_area( const VipsImage *image, const char *name, void **data )
  */
 void
 vips_image_set_blob( VipsImage *image, const char *name, 
-	VipsCallbackFn free_fn, void *data, size_t length )
+	VipsCallbackFn free_fn, const void *data, size_t size )
 {
 	GValue value = { 0 };
 
 	g_value_init( &value, VIPS_TYPE_BLOB );
-	vips_value_set_blob( &value, free_fn, data, length );
+	vips_value_set_blob( &value, free_fn, data, size );
 	vips_image_set( image, name, &value );
 	g_value_unset( &value );
 }
 
+/** 
+ * vips_image_set_blob_copy: (method)
+ * @image: image to attach the metadata to
+ * @name: metadata name
+ * @data: pointer to area of memory
+ * @length: length of memory area
+ *
+ * Attaches @blob as a metadata item on @image under the name @name, taking
+ * a copy of the memory area. A convenience function over 
+ * vips_image_set_blob().
+ *
+ * See also: vips_image_get_blob(), vips_image_set().
+ */
+void
+vips_image_set_blob_copy( VipsImage *image, 
+	const char *name, const void *data, size_t length )
+{
+	void *data_copy;
+
+	/* Cap at 100mb for sanity.
+	 */
+	if( !data ||
+		length == 0 ||
+		length > 100 * 1024 * 1024 )
+		return;
+
+	/* We add an extra, secret null byte at the end, just in case this blob 
+	 * is read as a C string. The libtiff reader attaches
+	 * XMP XML as a blob, for example.
+	 */
+	if( !(data_copy = vips_malloc( NULL, length + 1 )) ) 
+		return;
+	memcpy( data_copy, data, length );
+	((unsigned char *) data_copy)[length] = '\0';
+
+	vips_image_set_blob( image, 
+		name, (VipsCallbackFn) vips_free, data_copy, length );
+}
+
 /** 
  * vips_image_get_blob: (method)
  * @image: image to get the metadata from
@@ -1368,7 +1481,7 @@ vips_image_set_blob( VipsImage *image, const char *name,
  * Gets @blob from @image under the name @name, optionally returns its length in
  * @length. A convenience
  * function over vips_image_get(). Use vips_image_get_typeof() to test for the 
- * existance
+ * existence
  * of a piece of metadata.
  *
  * See also: vips_image_get(), vips_image_get_typeof(), vips_blob_get(), 
@@ -1377,7 +1490,7 @@ vips_image_set_blob( VipsImage *image, const char *name,
  */
 int
 vips_image_get_blob( const VipsImage *image, const char *name, 
-	void **data, size_t *length )
+	const void **data, size_t *length )
 {
 	GValue value_copy = { 0 };
 
@@ -1498,7 +1611,7 @@ vips_image_set_double( VipsImage *image, const char *name, double d )
  *
  * Gets @out from @im under the name @name. 
  * The field must be of type
- * VIPS_TYPE_REFSTRING.
+ * G_STRING, VIPS_TYPE_REFSTRING.
  *
  * Do not free @out.
  *
@@ -1513,21 +1626,29 @@ vips_image_get_string( const VipsImage *image, const char *name,
 	const char **out )
 {
 	GValue value = { 0 };
-	VipsArea *area;
 
 	if( vips_image_get( image, name, &value ) )
 		return( -1 );
-	if( G_VALUE_TYPE( &value ) != VIPS_TYPE_REF_STRING ) {
+
+	if( G_VALUE_TYPE( &value ) == VIPS_TYPE_REF_STRING ) {
+		VipsArea *area;
+
+		area = g_value_get_boxed( &value );
+		*out = area->data;
+	}
+	else if( G_VALUE_TYPE( &value ) == G_TYPE_STRING ) {
+		*out = g_value_get_string( &value );
+	}
+	else {
 		vips_error( "VipsImage",
 			_( "field \"%s\" is of type %s, not VipsRefString" ),
 			name,
 			g_type_name( G_VALUE_TYPE( &value ) ) );
 		g_value_unset( &value );
+
 		return( -1 );
 	}
 
-	area = g_value_get_boxed( &value );
-	*out = area->data;
 	g_value_unset( &value );
 
 	return( 0 );
@@ -1541,9 +1662,9 @@ vips_image_get_string( const VipsImage *image, const char *name,
  *
  * Attaches @str as a metadata item on @image as @name. 
  * A convenience
- * function over vips_image_set() using an vips_ref_string.
+ * function over vips_image_set() using #VIPS_TYPE_REF_STRING.
  *
- * See also: vips_image_get_double(), vips_image_set(), vips_ref_string
+ * See also: vips_image_get_double(), vips_image_set().
  */
 void
 vips_image_set_string( VipsImage *image, const char *name, const char *str )
@@ -1626,6 +1747,59 @@ vips_image_print_field( const VipsImage *image, const char *name )
 	g_free( str ); 
 }
 
+/** 
+ * vips_image_get_image: (method)
+ * @image: image to get the metadata from
+ * @name: metadata name
+ * @out: (transfer full): return metadata value
+ *
+ * Gets @out from @im under the name @name. 
+ * The field must be of type
+ * #VIPS_TYPE_IMAGE. You must unref @out with g_object_unref().
+ *
+ * Use vips_image_get_typeof() to test for the 
+ * existence of a piece of metadata.
+ *
+ * See also: vips_image_get(), vips_image_set_image()
+ *
+ * Returns: 0 on success, -1 otherwise.
+ */
+int 
+vips_image_get_image( const VipsImage *image, 
+	const char *name, VipsImage **out )
+{
+	GValue value = { 0 };
+
+	if( meta_get_value( image, name, VIPS_TYPE_IMAGE, &value ) ) 
+		return( -1 ); 
+	*out = g_value_dup_object( &value );
+	g_value_unset( &value );
+
+	return( 0 );
+}
+
+/**
+ * vips_image_set_image: (method)
+ * @image: image to attach the metadata to
+ * @name: metadata name
+ * @im: metadata value
+ *
+ * Attaches @im as a metadata item on @image as @name. 
+ * A convenience function over vips_image_set().
+ *
+ * See also: vips_image_get_image(), vips_image_set().
+ */
+void 
+vips_image_set_image( VipsImage *image, const char *name, VipsImage *im )
+{
+	GValue value = { 0 };
+
+	g_value_init( &value, VIPS_TYPE_IMAGE );
+	g_value_set_object( &value, im );
+	vips_image_set( image, name, &value );
+	g_value_unset( &value );
+}
+
 /**
  * vips_image_history_printf: (method)
  * @image: add history line to this image
