@@ -25,6 +25,8 @@
  * 	- use O_TMPFILE, if available
  * 23/7/18
  * 	- escape ASCII control characters in XML
+ * 29/8/19
+ * 	- verify bands/format for coded images
  */
 
 /*
@@ -137,7 +139,6 @@
  *
  * We use O_RDWR not O_WRONLY since after writing we may want to rewind the 
  * image and read from it.
- *
  */
 #define MODE_WRITE BINARYIZE (O_RDWR | O_CREAT | O_TRUNC)
 
@@ -349,8 +350,8 @@ vips__read_header_bytes( VipsImage *im, unsigned char *from )
 	from += 4;
 	if( im->magic != VIPS_MAGIC_INTEL && 
 		im->magic != VIPS_MAGIC_SPARC ) {
-		vips_error( "VipsImage", _( "\"%s\" is not a VIPS image" ), 
-			im->filename );
+		vips_error( "VipsImage", 
+			_( "\"%s\" is not a VIPS image" ), im->filename );
 		return( -1 );
 	}
 
@@ -384,10 +385,51 @@ vips__read_header_bytes( VipsImage *im, unsigned char *from )
 	im->Bands = VIPS_CLIP( 1, im->Bands, VIPS_MAX_COORD );
 	im->BandFmt = VIPS_CLIP( 0, im->BandFmt, VIPS_FORMAT_LAST - 1 );
 
-	/* Type, Coding, Offset, Res, etc. don't affect vips file layout, just 
+	/* Coding and Type have missing values, so we look up in the enum.
+	 */
+	im->Type = g_enum_get_value( 
+			g_type_class_ref( VIPS_TYPE_INTERPRETATION ), 
+			im->Type ) ?
+		im->Type : VIPS_INTERPRETATION_ERROR;
+	im->Coding = g_enum_get_value( 
+			g_type_class_ref( VIPS_TYPE_CODING ), 
+			im->Coding ) ?
+		im->Coding : VIPS_CODING_ERROR;
+
+	/* Offset, Res, etc. don't affect vips file layout, just 
 	 * pixel interpretation, don't clip them.
 	 */
 
+	/* Coding values imply Bands and BandFmt settings --- make sure they
+	 * are sane.
+	 */
+	switch( im->Coding ) {
+	case VIPS_CODING_NONE:
+		break;
+
+	case VIPS_CODING_LABQ:
+		if( im->Bands != 4 ||
+			im->BandFmt != VIPS_FORMAT_UCHAR ) {
+			vips_error( "VipsImage", 
+				"%s", _( "malformed LABQ image" ) ); 
+			return( -1 );
+		}
+		break;
+
+	case VIPS_CODING_RAD:
+		if( im->Bands != 4 ||
+			im->BandFmt != VIPS_FORMAT_UCHAR ) {
+			vips_error( "VipsImage", 
+				"%s", _( "malformed RAD image" ) ); 
+			return( -1 );
+		}
+		break;
+
+	default:
+		g_assert_not_reached();
+		break;
+	}
+
 	return( 0 );
 }
 
@@ -785,44 +827,6 @@ dbuf_write_quotes( VipsDbuf *dbuf, const char *str )
 	}
 }
 
-/* Append a string to a buffer, but escape stuff that xml hates in text. Our
- * argument string is utf-8.
- *
- * XML rules:
- *
- * 	We must escape &<> 
- * 	Don't escape \n, \t, \r
- * 	Do escape the other ASCII codes. 
- */
-static void
-dbuf_write_amp( VipsDbuf *dbuf, const char *str )
-{
-	const char *p;
-
-	for( p = str; *p; p++ ) 
-		if( *p < 32 &&
-			*p != '\n' &&
-			*p != '\t' &&
-			*p != '\r' )
-			/* You'd think we could output "&#x02%x;", but xml
-			 * 1.0 parsers barf on that. xml 1.1 allows this, but
-			 * there are almost no parsers. 
-			 *
-			 * U+2400 onwards are unicode glyphs for the ASCII 
-			 * control characters, so we can use them -- thanks
-			 * electroly.
-			 */
-			vips_dbuf_writef( dbuf, "&#x%04x;", 0x2400 + *p ); 
-		else if( *p == '<' )
-			vips_dbuf_write( dbuf, (guchar *) "&lt;", 4 );
-		else if( *p == '>' )
-			vips_dbuf_write( dbuf, (guchar *) "&gt;", 4 );
-		else if( *p == '&' )
-			vips_dbuf_write( dbuf, (guchar *) "&amp;", 5 );
-		else 
-			vips_dbuf_write( dbuf, (guchar *) p, 1 );
-}
-
 static void *
 build_xml_meta( VipsMeta *meta, VipsDbuf *dbuf )
 {
@@ -854,7 +858,7 @@ build_xml_meta( VipsMeta *meta, VipsDbuf *dbuf )
 				g_type_name( type ) ); 
 			dbuf_write_quotes( dbuf, meta->name );
 			vips_dbuf_writef( dbuf, "\">" );  
-			dbuf_write_amp( dbuf, str );
+			vips_dbuf_write_amp( dbuf, str );
 			vips_dbuf_writef( dbuf, "</field>\n" );  
 		}
 
@@ -885,7 +889,7 @@ build_xml( VipsImage *image )
 		vips_dbuf_writef( &dbuf, 
 			"    <field type=\"%s\" name=\"Hist\">", 
 			g_type_name( VIPS_TYPE_REF_STRING ) );
-		dbuf_write_amp( &dbuf, str );
+		vips_dbuf_write_amp( &dbuf, str );
 		vips_dbuf_writef( &dbuf, "</field>\n" ); 
 	}
 
@@ -930,11 +934,11 @@ vips__xml_properties_meta( VipsImage *image,
 
 		vips_dbuf_writef( dbuf, "    <property>\n" );  
 		vips_dbuf_writef( dbuf, "      <name>" ); 
-		dbuf_write_amp( dbuf, field );
+		vips_dbuf_write_amp( dbuf, field );
 		vips_dbuf_writef( dbuf, "</name>\n" ); 
 		vips_dbuf_writef( dbuf, "      <value type=\"%s\">",
 			g_type_name( type ) );  
-		dbuf_write_amp( dbuf, str );
+		vips_dbuf_write_amp( dbuf, str );
 		vips_dbuf_writef( dbuf, "</value>\n" ); 
 		vips_dbuf_writef( dbuf, "    </property>\n" );  
 
