@@ -1,5 +1,6 @@
 # vim: set fileencoding=utf-8 :
-import gc
+
+import sys
 import os
 import shutil
 import tempfile
@@ -10,7 +11,7 @@ from helpers import \
     JPEG_FILE, SRGB_FILE, MATLAB_FILE, PNG_FILE, TIF_FILE, OME_FILE, \
     ANALYZE_FILE, GIF_FILE, WEBP_FILE, EXR_FILE, FITS_FILE, OPENSLIDE_FILE, \
     PDF_FILE, SVG_FILE, SVGZ_FILE, SVG_GZ_FILE, GIF_ANIM_FILE, DICOM_FILE, \
-    BMP_FILE, NIFTI_FILE, ICO_FILE, \
+    BMP_FILE, NIFTI_FILE, ICO_FILE, HEIC_FILE, \
     temp_filename, assert_almost_equal_objects, have, skip_if_no
 
 
@@ -232,7 +233,6 @@ class TestForeign:
             # format area at the end
             assert y.startswith("hello world")
 
-
     @skip_if_no("pngload")
     def test_png(self):
         def png_valid(im):
@@ -247,6 +247,8 @@ class TestForeign:
         self.save_load_buffer("pngsave_buffer", "pngload_buffer", self.colour)
         self.save_load("%s.png", self.mono)
         self.save_load("%s.png", self.colour)
+        self.save_load_file(".png", "[interlace]", self.colour, 0)
+        self.save_load_file(".png", "[interlace]", self.mono, 0)
 
     @skip_if_no("tiffload")
     def test_tiff(self):
@@ -386,9 +388,9 @@ class TestForeign:
         self.buffer_loader("magickload_buffer", BMP_FILE, bmp_valid)
 
         # we should have rgb or rgba for svg files ... different versions of
-        # IM handle this differently
+        # IM handle this differently. GM even gives 1 band.
         im = pyvips.Image.magickload(SVG_FILE)
-        assert im.bands == 3 or im.bands == 4
+        assert im.bands == 3 or im.bands == 4 or im.bands == 1
 
         # density should change size of generated svg
         im = pyvips.Image.magickload(SVG_FILE, density='100')
@@ -400,12 +402,10 @@ class TestForeign:
         # assert im.width == width * 2
         # assert im.height == height * 2
 
-        # all-frames should load every frame of the animation
-        # (though all-frames is deprecated)
         im = pyvips.Image.magickload(GIF_ANIM_FILE)
         width = im.width
         height = im.height
-        im = pyvips.Image.magickload(GIF_ANIM_FILE, all_frames=True)
+        im = pyvips.Image.magickload(GIF_ANIM_FILE, n=-1)
         assert im.width == width
         assert im.height == height * 5
 
@@ -428,12 +428,6 @@ class TestForeign:
         # some IMs are 3 bands, some are 1, can't really test
         # assert im.bands == 1
 
-        # added in 8.7
-        self.save_load_file(".bmp", "", self.colour, 0)
-        self.save_load_buffer("magicksave_buffer", "magickload_buffer",
-                              self.colour, 0, format="BMP")
-        self.save_load("%s.bmp", self.colour)
-
         # libvips has its own sniffer for ICO, test that
         with open(ICO_FILE, 'rb') as f:
             buf = f.read()
@@ -442,11 +436,34 @@ class TestForeign:
         assert im.width == 16
         assert im.height == 16
 
+    # added in 8.7
+    @skip_if_no("magicksave")
+    def test_magicksave(self):
+        # save to a file and load again ... we can't use save_load_file since
+        # we want to make sure we use magickload/save 
+        # don't use BMP - GraphicsMagick always adds an alpha
+        # don't use TIF - IM7 will save as 16-bit
+        filename = temp_filename(self.tempdir, ".jpg")
+
+        self.colour.magicksave(filename)
+        x = pyvips.Image.magickload(filename)
+
+        assert self.colour.width == x.width
+        assert self.colour.height == x.height
+        assert self.colour.bands == x.bands
+        max_diff = (self.colour - x).abs().max()
+        assert max_diff < 60
+
+        self.save_load_buffer("magicksave_buffer", "magickload_buffer",
+                              self.colour, 60, format="JPG")
+
     @skip_if_no("webpload")
     def test_webp(self):
         def webp_valid(im):
             a = im(10, 10)
-            assert_almost_equal_objects(a, [71, 166, 236])
+            # different webp versions use different rounding systems leading
+            # to small variations
+            assert_almost_equal_objects(a, [71, 166, 236], threshold=2)
             assert im.width == 550
             assert im.height == 368
             assert im.bands == 3
@@ -461,7 +478,7 @@ class TestForeign:
         im = pyvips.Image.new_from_file(WEBP_FILE)
         buf = im.webpsave_buffer(lossless=True)
         im2 = pyvips.Image.new_from_buffer(buf, "")
-        assert im.avg() == im2.avg()
+        assert abs(im.avg() - im2.avg()) < 1
 
         # higher Q should mean a bigger buffer
         b1 = im.webpsave_buffer(Q=10)
@@ -492,6 +509,18 @@ class TestForeign:
                 y = pyvips.Image.new_from_buffer(buf, "")
                 assert y.get("orientation") == 6
 
+        # try converting an animated gif to webp ... can't do back to gif
+        # again without IM support
+        if have("gifload"):
+            x1 = pyvips.Image.new_from_file(GIF_ANIM_FILE, n=-1)
+            w1 = x1.webpsave_buffer(Q=10)
+            x2 = pyvips.Image.new_from_buffer(w1, "", n=-1)
+            assert x1.width == x2.width
+            assert x1.height == x2.height
+            assert x1.get("gif-delay") == x2.get("gif-delay")
+            assert x1.get("page-height") == x2.get("page-height")
+            assert x1.get("gif-loop") == x2.get("gif-loop")
+
     @skip_if_no("analyzeload")
     def test_analyzeload(self):
         def analyze_valid(im):
@@ -549,13 +578,6 @@ class TestForeign:
             assert im.height == 9919
             assert im.bands == 1
 
-        print("NIFTI_FILE =", NIFTI_FILE)
-        im = pyvips.Operation.call("niftiload", NIFTI_FILE)
-        print("width = ", im.width)
-
-        im = pyvips.Operation.call("niftiload", "test/images/avg152T1_LR_nifti.nii.gz")
-        print("width = ", im.width)
-
         self.file_loader("niftiload", NIFTI_FILE, nifti_valid)
         self.save_load("%s.nii.gz", self.mono)
 
@@ -575,7 +597,7 @@ class TestForeign:
         def pdf_valid(im):
             a = im(10, 10)
             assert_almost_equal_objects(a, [35, 31, 32, 255])
-            assert im.width == 1133
+            assert im.width == 1134
             assert im.height == 680
             assert im.bands == 4
 
@@ -622,15 +644,9 @@ class TestForeign:
     def test_svgload(self):
         def svg_valid(im):
             a = im(10, 10)
-
-            # some old rsvg versions are way, way off
-            assert abs(a[0] - 79) < 2
-            assert abs(a[1] - 79) < 2
-            assert abs(a[2] - 132) < 2
-            assert abs(a[3] - 255) < 2
-
-            assert im.width == 288
-            assert im.height == 470
+            assert_almost_equal_objects(a, [0, 0, 0, 0])
+            assert im.width == 736
+            assert im.height == 552
             assert im.bands == 4
 
         self.file_loader("svgload", SVG_FILE, svg_valid)
@@ -810,5 +826,61 @@ class TestForeign:
         buf = self.colour.dzsave_buffer(region_shrink="mode")
         buf = self.colour.dzsave_buffer(region_shrink="median")
 
+    @skip_if_no("heifload")
+    def test_heifload(self):
+        def heif_valid(im):
+            a = im(10, 10)
+            assert_almost_equal_objects(a, [75.0, 86.0, 81.0], threshold=2)
+            assert im.width == 4032
+            assert im.height == 3024
+            assert im.bands == 3
+
+        self.file_loader("heifload", HEIC_FILE, heif_valid)
+        self.buffer_loader("heifload_buffer", HEIC_FILE, heif_valid)
+
+    @skip_if_no("heifsave")
+    def test_heifsave(self):
+        self.save_load_buffer("heifsave_buffer", "heifload_buffer",
+                              self.colour, 80)
+        self.save_load("%s.heic", self.colour)
+
+        # test lossless mode
+        im = pyvips.Image.new_from_file(HEIC_FILE)
+        buf = im.heifsave_buffer(lossless=True)
+        im2 = pyvips.Image.new_from_buffer(buf, "")
+        # not in fact quite lossless
+        assert abs(im.avg() - im2.avg()) < 3
+
+        # higher Q should mean a bigger buffer
+        b1 = im.heifsave_buffer(Q=10)
+        b2 = im.heifsave_buffer(Q=90)
+        assert len(b2) > len(b1)
+
+        # try saving an image with an ICC profile and reading it back 
+        # not all libheif have profile support, so put it in an if
+        buf = self.colour.heifsave_buffer()
+        im = pyvips.Image.new_from_buffer(buf, "")
+        p1 = self.colour.get("icc-profile-data")
+        if im.get_typeof("icc-profile-data") != 0:
+            p2 = im.get("icc-profile-data")
+            assert p1 == p2
+
+        # add tests for exif, xmp, ipct
+        # the exif test will need us to be able to walk the header,
+        # we can't just check exif-data
+
+        # libheif 1.1 (on ubuntu 18.04, current LTS) does not support exif
+        # write, so this test is commented out
+
+        # test that exif changes change the output of heifsave
+        # first make sure we have exif support
+        #z = pyvips.Image.new_from_file(JPEG_FILE)
+        #if z.get_typeof("exif-ifd0-Orientation") != 0:
+        #    x = self.colour.copy()
+        #    x.set("exif-ifd0-Make", "banana")
+        #    buf = x.heifsave_buffer()
+        #    y = pyvips.Image.new_from_buffer(buf, "")
+        #    assert y.get("exif-ifd0-Make").split(" ")[0] == "banana"
+
 if __name__ == '__main__':
     pytest.main()
